# Simple OAuth Token Exchange

## Contents of this file

- [Introduction](#introduction)
- [Requirements](#requirements)
- [Installation](#installation)
- [Configuration](#configuration)
- [Maintainers](#maintainers)

## Introduction

This module extends the Simple OAuth module to provide OAuth 2.0 Token Exchange
functionality as specified in [RFC 8693](https://datatracker.ietf.org/doc/html/rfc8693).

For now, it allows clients to exchange tokens for tokens with different scopes.

### For what is this for?

When you need to implement token exchange scenarios in your OAuth 2.0 setup,
such as:

- Exchange existing access token for a new one with different scopes
- When needing access in the frontend application in a different context with
  narrower scoped (e.g. Delegate from [BFF](https://datatracker.ietf.org/doc/html/draft-ietf-oauth-browser-based-apps#name-backend-for-frontend-bff) to the browser)

The module implements the token exchange grant type, allowing
clients to request new tokens by presenting existing tokens as
proof of authorization.

### Functionality

The following features of [RFC 8693](https://datatracker.ietf.org/doc/html/rfc8693) are implemented:

- [x] Token Exchange grant type
- [x] Access Token as `subject_token`
- [x] The `scope` request parameter
- [ ] The `resource` request parameter
- [ ] The `audience` request parameter
- [ ] The `actor_token` request parameter
- [ ] The `actor_token_type` request parameter
- [ ] The `requested_token_type` request parameter
- [ ] Cross-client token exchange
- [ ] Support for issuing Refresh Tokens via token exchange

> This list gives an overview of what features are implemented
> from the RFC.

The module provides a robust implemenmtation for handling token
exchange requests while maintaining security and compliance with
the OAuth 2.0 specification.

### Security considerations

The module does it's best to ensure maximum security.
Key points:

- The `subject_token` must be an access token issued by the same client
  that is making the token exchange request.
- The `subject_token` must be a valid access token.
- The `scope` request parameter must be a subset of the scopes granted to
  the client making the token exchange request.
- If `scope` is not provided, the granted scopes on the `subject_token`
  are used.

> **Note:** Keep in mind that even if the requested scopes of the exchanged
> token are narrow, in the context of Drupal the `authenticated user` role
> is always in effect!

> **Warning:** It is up to the developer to ensure that the exchanged token
> is used in a way that is compliant with the requested scopes.

## Requirements

This module requires Drupal 10.3 or 11 and the following contrib modules:

- [Simple OAuth](https://www.drupal.org/project/simple_oauth)

## Installation

Install as you would normally install a contributed Drupal module. For further
information, see
[Installing Drupal Modules](https://www.drupal.org/docs/extending-drupal/installing-drupal-modules).

## Configuration

Configuration is primarily handled through the Simple OAuth module's settings.
Additional configuration may be required depending on your specific token
exchange requirements.

## Maintainers

Current maintainers:

- Christoph Niedermoser ([@nimoatwoodway](https://www.drupal.org/u/nimoatwoodway))
- Christian Foidl ([@chfoidl](https://www.drupal.org/u/chfoidl))
