<?php

declare(strict_types=1);

namespace Drupal\simple_oauth_token_exchange\Plugin\Oauth2Grant;

use Drupal\consumers\Entity\Consumer;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\simple_oauth\Plugin\Oauth2GrantBase;
use Drupal\simple_oauth_token_exchange\Grant\TokenExchangeGrant;
use Drupal\simple_oauth_token_exchange\Service\SubjectTokenManagerFactory;
use League\OAuth2\Server\Grant\GrantTypeInterface;
use League\OAuth2\Server\Repositories\RefreshTokenRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * The OAuth 2.0 Token Exchange grant plugin.
 *
 * @Oauth2Grant(
 *   id = "token_exchange",
 *   label = @Translation("Token Exchange")
 * )
 */
class TokenExchange extends Oauth2GrantBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected readonly RefreshTokenRepositoryInterface $refreshTokenRepository,
    protected readonly SubjectTokenManagerFactory $subjectTokenManagerFactory,
    protected readonly RequestStack $requestStack,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('simple_oauth.repositories.refresh_token'),
      $container->get(SubjectTokenManagerFactory::class),
      $container->get(RequestStack::class),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getGrantType(Consumer $client): GrantTypeInterface {
    return new TokenExchangeGrant($this->refreshTokenRepository, $this->subjectTokenManagerFactory, $this->requestStack);
  }

}
