<?php

declare(strict_types=1);

namespace Drupal\simple_oauth_token_exchange\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\simple_oauth\Server\ResourceServerFactoryInterface;
use League\OAuth2\Server\Repositories\ScopeRepositoryInterface;
use League\OAuth2\Server\ResourceServer;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Factory for creating subject token managers.
 */
class SubjectTokenManagerFactory {

  /**
   * The resource server.
   */
  protected readonly ResourceServer $resourceServer;

  /**
   * Constructs a new SubjectTokenManagerFactory.
   */
  public function __construct(
    #[Autowire(service: 'simple_oauth.server.resource_server.factory')]
    ResourceServerFactoryInterface $resourceServerFactory,
    #[Autowire(service: 'simple_oauth.repositories.scope')]
    protected readonly ScopeRepositoryInterface $scopeRepository,
    protected readonly EntityTypeManagerInterface $entityTypeManager,
  ) {
    $this->resourceServer = $resourceServerFactory->get();
  }

  /**
   * Creates a new SubjectTokenManager.
   */
  public function create(string $tokenValue): SubjectTokenManager {
    return new SubjectTokenManager(
      $tokenValue,
      $this->resourceServer,
      $this->scopeRepository,
      $this->entityTypeManager,
    );
  }

}
