<?php

declare(strict_types=1);

namespace Drupal\simple_oauth_token_exchange;

/**
 * Contains token type identifiers as defined in RFC 8693 section 3.
 *
 * @see https://datatracker.ietf.org/doc/html/rfc8693#section-3
 */
final class TokenTypeIdentifiers {

  /**
   * An OAuth 2.0 access token.
   *
   * @var string
   */
  public const ACCESS_TOKEN = 'urn:ietf:params:oauth:token-type:access_token';

  /**
   * An OAuth 2.0 refresh token.
   *
   * @var string
   */
  public const REFRESH_TOKEN = 'urn:ietf:params:oauth:token-type:refresh_token';

  /**
   * A SAML 2.0 assertion.
   *
   * @var string
   */
  public const SAML2 = 'urn:ietf:params:oauth:token-type:saml2';

  /**
   * A SAML 1.1 assertion.
   *
   * @var string
   */
  public const SAML1 = 'urn:ietf:params:oauth:token-type:saml1';

  /**
   * Prevent instantiation of this class.
   */
  private function __construct() {}

}
