<?php

declare(strict_types=1);

namespace Drupal\Tests\simple_oauth_token_exchange\Functional;

use Drupal\consumers\Entity\Consumer;
use Drupal\simple_oauth\Entity\Oauth2Scope;
use Drupal\Tests\simple_oauth\Functional\TokenBearerFunctionalTestBase;

/**
 * Base class for token exchange functional tests.
 */
abstract class TokenExchangeFunctionalTestBase extends TokenBearerFunctionalTestBase {


  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'image',
    'options',
    'serialization',
    'simple_oauth_password_grant',
    'simple_oauth_token_exchange',
    'simple_oauth_test',
    'text',
    'user',
  ];

  /**
   * Creates a new OAuth2 client with the specified grant types.
   *
   * @param array $grantTypes
   *   The grant types to enable for this client.
   *
   * @return \Drupal\consumers\Entity\Consumer
   *   The created client.
   */
  protected function createClient(array $grantTypes): Consumer {
    $client = Consumer::create([
      'client_id' => $this->randomString(),
      'label' => $this->getRandomGenerator()->name(),
      'secret' => $this->clientSecret,
      'grant_types' => $grantTypes,
      'redirect' => [$this->redirectUri],
    ]);
    $client->save();

    return $client;
  }

  /**
   * Creates a new OAuth2 scope.
   *
   * @param string $name
   *   The scope name.
   * @param array $grantTypes
   *   The grant types to enable for this scope.
   *
   * @return \Drupal\simple_oauth\Entity\Oauth2Scope
   *   The created scope.
   */
  protected function createScope(string $name, array $grantTypes): Oauth2Scope {
    $grantTypeConfig = [];
    foreach ($grantTypes as $grantType) {
      $grantTypeConfig[$grantType] = [
        'status' => TRUE,
        'description' => "Scope {$name} for {$grantType} grant",
      ];
    }

    $scope = Oauth2Scope::create([
      'name' => $name,
      'description' => "Test scope {$name}",
      'grant_types' => $grantTypeConfig,
      'umbrella' => FALSE,
      'granularity' => 'permission',
      'permission' => 'access content',
    ]);
    $scope->save();

    return $scope;
  }

}
